/*
 * Truchet cubes
 *
 http://objects.povworld.org/cgi-bin/dl.cgi?fireplug.zip
 *
 * @todo	Migliorare il cubo di base (che taglio ci vuole per una giunzione ottimale?), in alternativa inserire raccordi opportuni
 * @todo	Telecamera adattiva alla scena
 */

#include "colors.inc"

global_settings {
  assumed_gamma 1.0
}

// atmospheric media can be generated by adding a media statement
// to the scene not attached to any specific object
media {    // atmospheric media sample
  intervals 10
  scattering { 1, rgb 0.03 }
  samples 1, 10
  confidence 0.9999
  variance 1/1000
  ratio 0.9
}

// ----------------------------------------

camera {
	location  <9, 12, -7>
	look_at   <3, 4, 4>
}

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  }
}
  /*
light_source {
  <0, 0, 0>            // light's position (translated below)
  color rgb <1, 1, 1>  // light's color
  translate <-30, 30, -30>
}   */

// ----------------------------------------

// Textures -------------------------------
#declare colored_hole=1;
#declare t_plane=texture{
	pigment{color rgb <0.3,0.5,0.7>}
	finish{ambient 0.5}
    normal{
        agate 0.105
    }
    //scale 0.01
}
#declare t_cube=texture{
	//pigment{rgb <1,0.5,0>}
	pigment{color rgb <0.1,0.1,0.1>}
    finish {  
    	ambient 0.1  
        specular 0.196
        roughness 0.4
        reflection 0.19
    }
    normal{
        agate 0.05
    }
    scale 0.01
}
#declare t_hole=texture{
	//pigment{rgb x}
	//pigment{rgb <1,0.5,0>}
	//pigment{rgb <0.8,0,0>}
	pigment{rgb <0,0,0>}
    finish {
    	ambient 0.35  
        specular 0.1396
        roughness 0.14
        reflection 0.139
    }
}
#declare t_something=texture{
	pigment{color rgb <0.1,0.1,0.1>}
    finish {  
    	ambient 0.1  
        specular 0.296
        roughness 0.4
        reflection 0.29
    }
}

// ----------------------------------------

// Objects --------------------------------
#declare cr=0.5;
#declare tr=0.06;
#declare trucube=difference {
    box{<-cr,-cr,-cr> <cr,cr,cr>}
    union {
        //Front
        torus{cr tr rotate 90*x translate <cr,cr,-cr>}
        torus{cr tr rotate 90*x translate <-cr,-cr,-cr>}
        //Rear
        torus{cr tr rotate 90*x translate <cr,-cr,cr>}
        torus{cr tr rotate 90*x translate <-cr,cr,cr>}
        //Top
        torus{cr tr translate <cr,cr,-cr>}
        torus{cr tr translate <-cr,cr,cr>}
        //Bottom
        torus{cr tr translate <cr,-cr,cr>}
        torus{cr tr translate <-cr,-cr,-cr>}
        //Left
        torus{cr tr rotate 90*z translate <-cr,cr,cr>}
        torus{cr tr rotate 90*z translate <-cr,-cr,-cr>}
        //Right
        torus{cr tr rotate 90*z translate <cr,cr,-cr>}
        torus{cr tr rotate 90*z translate <cr,-cr,cr>}
        #if(colored_hole)
            texture{t_hole}
        #end
    }
}
// ----------------------------------------

// Scene ----------------------------------
plane {y, -cr texture{t_plane}}

// The truchet-stair
#declare S1=seed(7);
#declare f_rot=0.3;

#local w_start=<-5,0,0>;
#local w_end=<5,9,10>;

#local w_pos=w_start;
#while(w_pos.x<w_end.x)
	#local w_pos=<w_pos.x,w_pos.y,w_start.z>;
	#while(w_pos.z<w_end.z)
		#local w_pos=<w_pos.x,w_start.y,w_pos.z>;
		#while(w_pos.y<w_end.y)
			#if(w_pos.y<=w_pos.z)
				//Random rotation
				#local x_rot=0;
				#if(rand(S1)<f_rot)
					#local x_rot=90;
				#end
				#local y_rot=0;
				#if(rand(S1)<f_rot)
					#local y_rot=90;
				#end
				#local z_rot=0;
				#if(rand(S1)<f_rot)
					#local z_rot=90;
				#end
				//Cube placement
				object {trucube texture{t_cube} rotate x_rot*x rotate y_rot*y rotate z_rot*z translate w_pos}
			#end
			#local w_pos=w_pos+y;
		#end
		#local w_pos=w_pos+z;
	#end
	#local w_pos=w_pos+x;
#end

// Something
//#local s_r=0.3;
#local something_pos=<w_start.x+1,w_start.y+5.5,w_start.z+5>;

//sphere{something_pos s_r texture{t_something}}
#include "fireplug.inc"
object{O_Fireplug scale 1/40 translate something_pos}



// Spotlights

/*
light_source {
  0*x                     // light's position (translated below)
  color rgb <1,1,1>       // light's color
  spotlight               // this kind of light source
  //translate <w_end.x*2,w_end.y,w_end.z/4> // <x y z> position of light
  //point_at <w_end.x,w_end.y/3,w_end.z/3*2>      // direction of spotlight
  translate <w_end.x,w_end.y,-w_end.z/2> // <x y z> position of light
  point_at something_pos      // direction of spotlight
  radius 5                // hotspot (inner, in degrees)
  tightness 50            // tightness of falloff (1...100) lower is softer, higher is tighter
  falloff 8               // intensity falloff radius (outer, in degrees)
}
*/
light_source {
  0*x                     // light's position (translated below)
  color rgb <1,1,1>       // light's color
  spotlight               // this kind of light source
  translate <w_start.x,w_end.y,-w_end.z/2> // <x y z> position of light
  point_at something_pos+0.5*y      // direction of spotlight
  radius 5                // hotspot (inner, in degrees)
  tightness 50            // tightness of falloff (1...100) lower is softer, higher is tighter
  falloff 8               // intensity falloff radius (outer, in degrees)
}

// ----------------------------------------

